from functools import wraps
from typing import Dict, Any, Callable
import httpx
from httpx import HTTPStatusError
from fastapi import APIRouter, Depends, status, HTTPException
from fastapi.responses import StreamingResponse, JSONResponse

from openjiuwen.core.common.logging import logger

from openjiuwen_studio.core.thirdparty_client import DeepSearchAgentClient
from openjiuwen_studio.core.manager.convertor.components.llm import get_model_config
from openjiuwen_studio.core.manager.login_manager.user import get_current_user
from openjiuwen_studio.core.manager.login_manager.space import check_user_space
from openjiuwen_studio.core.common.exceptions import DeepSearchClientError
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.deepsearch import (
    DeepSearchRequest,
    TemplateImportRequest,
    TemplateImportResponse,
    TemplateListResponse,
    TemplateGetResponse,
    TemplateDeleteResponse,
    TemplateUpdateRequest,
    TemplateUpdateResponse,
    WebSearchEngineCreateRes,
    WebSearchEngineCreateRequestDTO,
    WebSearchEngineGetRes,
    WebSearchEngineListRes,
    WebSearchEngineUpdateRes,
    WebSearchEngineUpdateRequestDTO,
    WebSearchEngineDeleteRes,
    WebSearchEngineAccessRequestDTO,
    WebSearchEngineAccessRes,
    ReportConvertReq,
    ReportConvertRes,
)

deepsearch_router = APIRouter()


# 依赖注入（或直接使用单例）
def get_agent_client():
    return DeepSearchAgentClient()  # 或全局单例


def build_single_model_config(model_id, space_id):
    """构建单个模型配置"""
    model_config = get_model_config(model_id, space_id)
    return {
        "model_name": model_config.model_type,
        "model_type": model_config.provider,
        "base_url": model_config.base_url,
        "api_key": model_config.api_key,
        "hyper_parameters": {
            "top_p": model_config.parameters.get("top_p"),
            "frequency_penalty": 0,
            "max_tokens": model_config.parameters.get("max_tokens"),
            "temperature": model_config.parameters.get("temperature"),
        }
    }


def get_model_configs(
    general_model_id,
    space_id,
    plan_understanding_model_id=None,
    info_collecting_model_id=None,
    writing_checking_model_id=None
):
    """构建 llm_config 结构，高级配置仅在有值时添加"""
    # llm_config = build_single_model_config(general_model_id, space_id)
    llm_config = {}
    llm_config["general"] = build_single_model_config(general_model_id, space_id)

    # 高级配置：仅在有值时添加
    if plan_understanding_model_id:
        llm_config["plan_understanding"] = build_single_model_config(plan_understanding_model_id, space_id)
    if info_collecting_model_id:
        llm_config["info_collecting"] = build_single_model_config(info_collecting_model_id, space_id)
    if writing_checking_model_id:
        llm_config["writing_checking"] = build_single_model_config(writing_checking_model_id, space_id)

    return llm_config


def handle_deepsearch_errors(func: Callable[..., Any]) -> Callable[..., Any]:
    """
    装饰器：自动捕获 HTTPStatusError 并返回通用错误响应。
    适用于返回 JSONResponse 或 dict 的非流式接口。
    """
    @wraps(func)
    async def wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except HTTPStatusError as exc:
            # 记录原始错误详情到服务器日志，便于排查问题
            logger.error(
                "DeepSearch service error: status=%s, body=%s",
                exc.response.status_code,
                exc.response.text[:1000] if exc.response.text else "",
            )
            # 状态码规范化：内部服务的5xx错误统一返回502，4xx错误保留原始状态码
            status_code = exc.response.status_code if 400 <= exc.response.status_code < 500 else 502
            return JSONResponse(
                status_code=status_code,
                content={"detail": "DeepSearch service request failed. Please try again later."},
            )
    return wrapper


@deepsearch_router.post("/run", response_model=ResponseModel[dict])
async def run(
        request: DeepSearchRequest,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
) -> StreamingResponse:
    # 使用 request.model_dump() 保留前端传递的所有字段（除了 *_model_config_id）
    payload = request.model_dump(
        exclude_none=True,
        exclude={
            'general_model_config_id',
            'plan_understanding_model_id',
            'info_collecting_model_id',
            'writing_checking_model_id',
        }
    )
    # 取消请求不需要获取模型配置，直接转发到 deepsearch 服务
    if request.interrupt_feedback == "cancel":
        logger.info(f"[DeepSearch Cancel] Received cancel request for conversation_id={payload.get('conversation_id')}")
        # 取消请求：不需要 llm_config，直接转发
        pass
    else:
        # 构建完整的 llm_config（包含 general, plan_understanding 等）
        model_config = get_model_configs(
            request.general_model_config_id,
            request.space_id,
            request.plan_understanding_model_id,
            request.info_collecting_model_id,
            request.writing_checking_model_id
        )
        # 用构建好的 model_config 覆盖 llm_config
        payload["llm_config"] = model_config

    _ = check_user_space(payload["space_id"], current_user)

    async def stream():
        try:
            async for line in client.run_deepsearch_stream(payload):
                if line:
                    yield line + "\n\n"
        except Exception as e:
            # 记录原始错误详情到服务器日志
            logger.error("DeepSearch client init error: %s", str(e))
            if isinstance(e, DeepSearchClientError):
                error = e
            else:
                error = DeepSearchClientError(
                    error_code="CLIENT_INIT_ERROR",
                    message="Failed to connect to DeepSearch service"
                )
            conversation_id = payload.get("conversation_id", "")
            for event_str in error.generate_error_stream(conversation_id):
                yield event_str

    return StreamingResponse(stream(), media_type="text/event-stream")


@deepsearch_router.post("/template", response_model=TemplateImportResponse)
@handle_deepsearch_errors
async def import_template(
        request: TemplateImportRequest,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """导入模板"""
    # 构建 llm_config（模板导入只需要 general 模型配置）
    model_config = get_model_configs(request.model_config_id, request.space_id)

    # 使用 request.model_dump() 保留前端传递的所有字段（除了 model_config_id）
    payload = request.model_dump(exclude={'model_config_id'})
    # 用构建好的 model_config 覆盖 llm_config
    payload["llm_config"] = model_config

    _ = check_user_space(payload["space_id"], current_user)
    result = await client.import_templates(payload)
    # 直接返回，FastAPI 会自动校验并序列化为 TemplateImportResponse
    return result


@deepsearch_router.get("/template/{space_id}", response_model=TemplateListResponse)
@handle_deepsearch_errors
async def list_templates(
        space_id: str,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Get template list by space_id"""
    _ = check_user_space(space_id, current_user)
    return await client.list_templates(space_id)


@deepsearch_router.get("/template/{space_id}/{template_id}", response_model=TemplateGetResponse)
@handle_deepsearch_errors
async def get_template(
        space_id: str,
        template_id: int,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Get template content by space_id and template_id"""
    _ = check_user_space(space_id, current_user)
    return await client.get_templates(space_id, template_id)


@deepsearch_router.delete("/template/{space_id}/{template_id}", response_model=TemplateDeleteResponse)
@handle_deepsearch_errors
async def delete_template(
        space_id: str,
        template_id: int,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Delete a specific template"""
    _ = check_user_space(space_id, current_user)
    return await client.delete_templates(space_id, template_id)


@deepsearch_router.put("/template", response_model=TemplateUpdateResponse)
@handle_deepsearch_errors
async def update_template(
        request: TemplateUpdateRequest,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Update a specific template"""
    payload = request.model_dump()
    _ = check_user_space(payload["space_id"], current_user)
    return await client.update_templates(payload)


@deepsearch_router.post("/web_search", response_model=WebSearchEngineCreateRes, status_code=status.HTTP_201_CREATED)
@handle_deepsearch_errors
async def create_web_search_engine(
        request: WebSearchEngineCreateRequestDTO,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Create a specific template"""
    payload = request.model_dump()
    _ = check_user_space(payload["space_id"], current_user)
    return await client.create_web_searchs(payload)


@deepsearch_router.get("/web_search/{space_id}/{web_search_engine_id}",
            response_model=WebSearchEngineGetRes, status_code=status.HTTP_200_OK)
@handle_deepsearch_errors
async def get_web_search_engine(
        space_id: str,
        web_search_engine_id: int,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Get web search by space_id and web_search_engine_id"""
    _ = check_user_space(space_id, current_user)
    return await client.get_web_search_engines(space_id, web_search_engine_id)


@deepsearch_router.get("/web_search/{space_id}",
            response_model=WebSearchEngineListRes, status_code=status.HTTP_200_OK)
@handle_deepsearch_errors
async def get_web_search_engine_list(
        space_id: str,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Get web search list"""
    _ = check_user_space(space_id, current_user)
    return await client.get_web_search_engine_lists(space_id)


@deepsearch_router.delete("/web_search/{space_id}/{web_search_engine_id}",
               response_model=WebSearchEngineDeleteRes, status_code=status.HTTP_200_OK)
@handle_deepsearch_errors
async def delete_web_search_engine(
        space_id: str,
        web_search_engine_id: int,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Delete web search by space_id and web_search_engine_id"""
    _ = check_user_space(space_id, current_user)
    return await client.delete_web_search_engines(space_id, web_search_engine_id)


@deepsearch_router.put("/web_search", response_model=WebSearchEngineUpdateRes, status_code=status.HTTP_200_OK)
@handle_deepsearch_errors
async def update_web_search_engine(
        request: WebSearchEngineUpdateRequestDTO,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """Update a specific web search"""
    payload = request.model_dump()
    _ = check_user_space(payload["space_id"], current_user)
    return await client.update_web_search_engines(payload)


@deepsearch_router.post("/web_search/{space_id}/{web_search_engine_id}",
             response_model=WebSearchEngineAccessRes, status_code=status.HTTP_201_CREATED)
@handle_deepsearch_errors
async def access_web_search_engine(
        space_id: str,
        web_search_engine_id: int,
        request: WebSearchEngineAccessRequestDTO,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """test web search"""
    payload = request.model_dump()
    _ = check_user_space(space_id, current_user)
    res = await client.access_web_search_engines(space_id, web_search_engine_id, payload)
    return res


@deepsearch_router.post("/reports/convert", response_model=ReportConvertRes)
@handle_deepsearch_errors
async def report_convert(
        request: ReportConvertReq,
        client: DeepSearchAgentClient = Depends(get_agent_client),
        current_user: dict = Depends(get_current_user)
):
    """转换生成的markdown报告的格式"""
    payload = request.model_dump()
    _ = check_user_space(payload["space_id"], current_user)
    return await client.report_converts(payload)


@deepsearch_router.get("/heartbeat")
async def deepsearch_heartbeat():
    """检查 DeepSearch 服务是否可用"""
    try:
        # 检查配置
        if not settings.deepsearch_agent_host or not settings.deepsearch_agent_port:
            return {
                "status": "unavailable",
                "message": "DeepSearch service not configured"
            }

        # 直接向 DeepSearch 服务发送健康检查请求
        base_url = f"http://{settings.deepsearch_agent_host}:{settings.deepsearch_agent_port}"
        async with httpx.AsyncClient(timeout=3.0) as client:
            response = await client.get(f"{base_url}/api/health")
            response.raise_for_status()

        # 检查响应内容
        data = response.json()
        if data.get("status") == "healthy":
            return {
                "status": "available",
                "message": "DeepSearch service is available"
            }
        else:
            return {
                "status": "unavailable",
                "message": "DeepSearch service is not healthy"
            }
    except httpx.ConnectError:
        logger.error("DeepSearch heartbeat: connection error")
        return {
            "status": "unavailable",
            "message": "Cannot connect to DeepSearch service"
        }
    except httpx.TimeoutException:
        logger.error("DeepSearch heartbeat: timeout")
        return {
            "status": "unavailable",
            "message": "DeepSearch service timeout"
        }
    except Exception as e:
        logger.error("DeepSearch heartbeat: %s", str(e))
        return {
            "status": "unavailable",
            "message": "DeepSearch service error"
        }



import io
import os
import sys
import time
from contextlib import asynccontextmanager

# 添加项目根目录到 Python 路径，以便直接运行时能找到所有模块
backend_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
if backend_dir not in sys.path:
    sys.path.insert(0, backend_dir)

import uvicorn
from dotenv import load_dotenv
from fastapi import FastAPI, Request, status
from fastapi.middleware.cors import CORSMiddleware
from starlette.middleware.base import BaseHTTPMiddleware
# Load environment variables from project root (上级目录)
project_root = os.path.dirname(backend_dir)
load_dotenv(os.path.join(project_root, '.env'))

from openjiuwen_studio.ops.modules.prompt.infra.database import create_database_tables
from openjiuwen_studio.memory_engine_start import MemoryEngineManager

from openjiuwen_studio.routers import register
from openjiuwen_studio.core.database import engine
from openjiuwen_studio.models.db_fun_base import Base
# Import all models to ensure they are registered with SQLAlchemy
from openjiuwen_studio.models import ModelConfig, ModelUsageLog, EmbeddingModelConfig, AgentBaseDB, AgentPublishDB, \
    PromptRelationDB, TagDB, UserDB, SpaceDB, SpaceUserDB, WorkflowBaseDB, WorkflowPublishDB, PluginBaseDB, \
    PluginPublishDB, ToolBaseDB, \
    WorkflowExecutionDB, WorkflowExecutionDetailsDB, AgentExecutionDB, AgentExecutionDetailsDB, \
    AgentWorkflowRelationDB, KnowledgeBaseDB, KnowledgeBaseDocumentDB, ReferenceDB, SystemEmbeddingModelDB, \
    SystemLLMModelDB, MemoryBaseDB
# Import alembic version check
from openjiuwen.core.common.logging import logger, interface_logger
from openjiuwen_studio.core.common.logging.events import CustomLogEventType
from openjiuwen_studio.core.alembic_version_check import check_alembic_versions
from openjiuwen_studio.ops.config import settings as ops_settings
from openjiuwen_studio.core.config import settings
# Import Trace models
from openjiuwen_studio.models.trace_detail import TraceDetailDB
from openjiuwen_studio.models.trace_summary import TraceSummaryDB
from openjiuwen_studio.models.tag import workflow_tag_association

from openjiuwen_studio.core.manager.redis_manager.redis_client import redis_manager_bytes
from openjiuwen.core.runner import Runner
from openjiuwen.core.runner.runner_config import get_runner_config
from openjiuwen.core.session.checkpointer.checkpointer import CheckpointerConfig

sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding='utf-8')


@asynccontextmanager
async def lifespan_func(app: FastAPI):
    # Startup
    logger.info("🚀 Starting Jiuwen Agent Studio Backend...")

    target_tables = [
        ModelConfig.__table__,
        ModelUsageLog.__table__,
        EmbeddingModelConfig.__table__,
        AgentBaseDB.__table__,
        AgentPublishDB.__table__,
        PromptRelationDB.__table__,
        TagDB.__table__,
        UserDB.__table__,
        SpaceDB.__table__,
        SpaceUserDB.__table__,
        WorkflowBaseDB.__table__,
        WorkflowPublishDB.__table__,
        PluginBaseDB.__table__,
        PluginPublishDB.__table__,
        ToolBaseDB.__table__,
        WorkflowExecutionDB.__table__,
        WorkflowExecutionDetailsDB.__table__,
        AgentExecutionDB.__table__,
        AgentExecutionDetailsDB.__table__,
        AgentWorkflowRelationDB.__table__,
        ReferenceDB.__table__,
        # Trace tables
        TraceDetailDB.__table__,
        TraceSummaryDB.__table__,
        # Knowledge Base tables
        KnowledgeBaseDB.__table__,
        KnowledgeBaseDocumentDB.__table__,
        # System model tables
        SystemLLMModelDB.__table__,
        SystemEmbeddingModelDB.__table__,
        # Memory Base tables
        MemoryBaseDB.__table__,
    ]

    if engine.url.drivername == "sqlite":
        renamed_count = 0
        for table in target_tables:
            # Skip if table has no index attribute
            if not hasattr(table, "indexes"):
                logger.warning(f"Table {table.name} has no indexes attribute, skipping...")
                continue
                # Iterate all indexes of the table
            for idx in table.indexes:
                old_idx_name = idx.name
                idx.name = f"{old_idx_name}_{table.name}"
                logger.info(f"{table.name}: Renamed index: {old_idx_name} ---> {idx.name}")
                renamed_count += 1
        logger.info(f"Duplicate index renaming completed. Total renamed indexes: {renamed_count}")

    # Create database tables with checkfirst=True to avoid creating existing indexes
    Base.metadata.create_all(
        bind=engine,
        tables=target_tables,
        checkfirst=True
    )
    await MemoryEngineManager.init()

    # Create workflow_tag_association table if it doesn't exist
    workflow_tag_association.create(bind=engine, checkfirst=True)

    # 检查 Alembic 版本
    check_alembic_versions()

    # ops数据库相关表自动创建
    create_database_tables()
    logger.info("✅ Database tables created")

    # 初始化 Runner
    if settings.enable_redis_checkpoint:
        runner_config = get_runner_config()
        runner_config.checkpointer_config = CheckpointerConfig(
            type="redis",
            conf={
                "connection": {
                    "redis_client": redis_manager_bytes.client
                }
            }
        )
        Runner.set_config(runner_config)

    await Runner.start()

    yield

    # Shutdown
    logger.info("🛑 Shutting down Jiuwen Agent Studio Backend...")


# Create FastAPI app
app = FastAPI(
    title="Jiuwen Agent Studio API",
    description="Backend API for Jiuwen Agent Studio - Professional LLM Agent Development Platform",
    version="1.0.0",
    docs_url="/api/docs",
    redoc_url="/api/redoc",
    openapi_url="/api/openapi.json",
    lifespan=lifespan_func,
    # 添加Swagger UI的OAuth2配置
    swagger_ui_init_oauth={
        "usePkceWithAuthorizationCodeGrant": True,
        "appName": "Jiuwen Agent Studio API",
        "clientId": "swagger-ui",
    }
)


class LogMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        start_time = time.perf_counter()
        try:
            response = await call_next(request)
        except Exception as e:
            interface_logger.error(
                "Request failed",
                event_type=CustomLogEventType.INTERFACE_SRV.value,
                interface_name=request.url.path,
                execution_time_ms=round((time.perf_counter() - start_time) * 1000),
                exception=str(e),
                error_code=str(status.HTTP_500_INTERNAL_SERVER_ERROR),
                metadata={
                    "route_path": request.url.path
                }
            )
            logger.error(f"Request failed: {e}")
            raise

        route_path = self._get_route_path(request)
        endpoint_name = self._truncate_path(route_path, max_parts=3)

        interface_logger.info(
            "",
            event_type=CustomLogEventType.INTERFACE_SRV.value,
            interface_name=endpoint_name,
            execution_time_ms=round((time.perf_counter() - start_time) * 1000),
            error_code=str(response.status_code),
            metadata={
                "route_path": route_path
            }
        )
        return response


    def _truncate_path(self, path: str, max_parts: int = 3) -> str:
        if not path or path == "/":
            return "/"
        parts = path.strip("/").split("/")
        truncated = parts[:max_parts]
        return "/" + "/".join(truncated) if truncated else "/"


    def _get_route_path(self, request: Request) -> str:
        try:
            # route 在 request.scope 中，由 Starlette 路由器在匹配后注入
            route = request.scope.get("route")
            if route and hasattr(route, "path") and isinstance(route.path, str):
                return route.path
            else:
                # 路由未匹配（如 404）或 route 对象异常
                return request.url.path
        except Exception:
            # 安全兜底：任何异常都返回原始路径
            return request.url.path


app.add_middleware(LogMiddleware)
# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["http://localhost:3000", "http://127.0.0.1:3000"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


@app.get("/")
async def root():
    return {
        "message": "Welcome to Jiuwen Agent Studio Backend",
        "docs": "/api/docs",
        "health": "/api/health"
    }


register.router_register(app)


def main():
    # Development configuration
    config = {
        "host": os.getenv("HOST", "0.0.0.0"),
        "port": int(os.getenv("BACKEND_PORT", "8000")),
        "reload": False,
        "log_level": "info",
        "access_log": True,
        "workers": int(os.getenv("WORKER_NUM", 1))
        if (os.getenv("INDEX_MANAGER_TYPE") == "milvus" and os.getenv("DB_TYPE") == "mysql")
        else 1,
    }

    logger.info("🚀 Starting Jiuwen Agent Studio Backend in development mode...")
    logger.info(f"📍 Server will be available at: http://{config['host']}:{config['port']}")
    logger.info(f"📚 API Documentation: http://{config['host']}:{config['port']}/api/docs")
    logger.info(f"🔍 Health Check: http://{config['host']}:{config['port']}/api/health")
    logger.info("🔄 Auto-reload enabled for development")
    logger.info("⏹️  Press Ctrl+C to stop the server")
    logger.info("-" * 60)

    # Start the server；force asyncio loop to avoid uvloop + nest_asyncio conflict
    uvicorn.run(
        "openjiuwen_studio.main:app",
        loop="asyncio",
        **config
    )


if __name__ == "__main__":
    main()



"""refactor_agent_model config

Revision ID: 69cb68688942
Revises: 031b34b4dd30
Create Date: 2026-01-31 16:52:49.151030

"""
import json
import logging
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import sqlite

from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists

# revision identifiers, used by Alembic.
revision: str = '69cb68688942'
down_revision: Union[str, Sequence[str], None] = '031b34b4dd30'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    # 1. 首先添加新列
    for table in ['agent', 'agent_publish']:
        with op.batch_alter_table(table, schema=None) as batch_op:
            if not column_exists(table, 'model_id'):
                batch_op.add_column(sa.Column('model_id', sa.Integer(), nullable=True))
            if not column_exists(table, 'agent_model_config'):
                batch_op.add_column(sa.Column('agent_model_config', sa.JSON(), nullable=True))

    # 2. 数据迁移
    conn = op.get_bind()
    for table in ['agent', 'agent_publish']:
        if column_exists(table, 'model'):
            try:
                res = conn.execute(sa.text(f"SELECT id, model FROM {table} WHERE model IS NOT NULL"))
                for row in res:
                    row_id = row[0]
                    model_data_str = row[1]
                    if not model_data_str:
                        continue
                    
                    try:
                        if isinstance(model_data_str, str):
                            model_data = json.loads(model_data_str)
                        else:
                            model_data = model_data_str
                        
                        model_info = model_data.get("model_info", {})
                        model_id_val = model_info.get("model_id")
                        
                        if model_id_val is not None:
                            # 提取特定的配置字段，保持与 AgentModelConfig 结构一致
                            config_to_save = {}
                            for field in ["temperature", "top_p", "timeout"]:
                                if field in model_info:
                                    config_to_save[field] = model_info[field]
                                    
                            conn.execute(
                                sa.text(
                                    f"UPDATE {table} SET model_id = :mid, "
                                    f"agent_model_config = :mconfig WHERE id = :id"
                                ),
                                {"mid": model_id_val, "mconfig": json.dumps(config_to_save), "id": row_id}
                            )
                    except Exception as e:
                        logging.error(f"Failed to migrate {table} ID {row_id}: {e}")
            except Exception as e:
                logging.error(f"Failed to query {table} for migration: {e}")

    # 3. 删除旧列
    for table in ['agent', 'agent_publish']:
        if column_exists(table, 'model'):
            with op.batch_alter_table(table, schema=None) as batch_op:
                batch_op.drop_column('model')
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    for table in ['agent_publish', 'agent']:
        with op.batch_alter_table(table, schema=None) as batch_op:
            if not column_exists(table, 'model'):
                batch_op.add_column(sa.Column('model', sqlite.JSON(), nullable=True))
            if column_exists(table, 'agent_model_config'):
                batch_op.drop_column('agent_model_config')
            if column_exists(table, 'model_id'):
                batch_op.drop_column('model_id')
    # ### end Alembic commands ###


from functools import wraps
from typing import Callable, List, Optional, Union

from fastapi import status
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import (
    get_db_jw, get_val_from_dict)
from openjiuwen_studio.models.user import SpaceDB, SpaceUserDB, UserDB
from openjiuwen_studio.schemas.common import ResponseModel


class UserRepository():
    def __init__(self) -> None:
        pass

    # def __init__(self, db: Session) -> None:
    #     self._user_db: JiuwenBaseRepository[UserDB] = JiuwenBaseRepository(db, UserDB)
    #     self._space_db: JiuwenBaseRepository[SpaceDB] = JiuwenBaseRepository(db, SpaceDB)
    #     self._space_user_db: JiuwenBaseRepository[SpaceUserDB] = JiuwenBaseRepository(db, SpaceUserDB)

    def with_exception_handling(func) -> Callable:
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                logger.error(f"Error: user&space db data preprocessing failed: {str(e)}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST, 
                    message=f"Error: user&space db data preprocessing failed: {str(e)}"
                ).model_dump(exclude_none=True)
        return wrapper

    '''
    description: 往数据库创建user数据
    param {dict} user_info  待创建数据, email要求必须要有
    return {*}
    '''
    @with_exception_handling
    def create_user_tbl(self, user_info: dict):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": get_val_from_dict(user_info, ["email"]),
            }
            return user_db.register_dl_in_sql(find_id=find_id, dl=user_info).model_dump(exclude_none=True)
    
    @with_exception_handling
    def update_user_tbl(self, user_info: dict):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": get_val_from_dict(user_info, ["email"]),
            }
            return user_db.update_dl_in_sql(find_id=find_id, update_dl=user_info).model_dump(exclude_none=True)
    
    '''
    description: 从数据库中查找user数据是否存在
    param {Optional} email  email和下面的session_key至少要有一个
    param {Optional} session_key
    param {int} role_type   
    return {*}
    '''
    @with_exception_handling
    def find_user_tbl(self, email: Optional[str] = None, session_key: Optional[str] = None):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": email,
                "session_key": session_key,
            }
            find_id = UserDB.filter_invalid_keys(find_id)
            return user_db._find_dl_in_sql(find_id=find_id).model_dump(exclude_none=True)
       
    '''
    description: 从数据库中获取user数据
    param {Optional} email  email和下面的session_key至少要有一个
    param {Optional} session_key
    param {int} role_type
    return {*}
    '''
    @with_exception_handling
    def get_user_tbl(self, email: Optional[str] = None, session_key: Optional[str] = None):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": email,
                "session_key": session_key,
            }
            find_id = UserDB.filter_invalid_keys(find_id)
            return user_db.get_dl_in_sql(find_id=find_id, return_first_item=True).model_dump(exclude_none=True)

    '''
    description: 更新数据库中user的session_key
    param {str} email   email作为健值定位user
    param {str} session_key
    return {*}
    '''
    @with_exception_handling
    def update_session_key(self, email: str, session_key: str):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": email,
            }
            update_data = {
                "session_key": session_key,
            }
            return user_db.update_dl_in_sql(find_id=find_id, update_dl=update_data).model_dump(exclude_none=True)
    
    '''
    description: 更新数据库中user的refresh_token
    param {str} email   email作为健值定位user
    param {str} refresh_token
    return {*}
    '''
    @with_exception_handling
    def update_refresh_token(self, email: str, refresh_token: str):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": email,
            }
            update_data = {
                "refresh_token": refresh_token,
            }
            return user_db.update_dl_in_sql(find_id=find_id, update_dl=update_data).model_dump(exclude_none=True)

    '''
    description: 删除数据库中的user
    param {Optional} email  email和下面的session_key至少要有一个
    param {Optional} session_key
    param {int} role_type
    return {*}
    '''
    @with_exception_handling
    def delete_user_tbl(self, email: Optional[str] = None, session_key: Optional[str] = None, role_type: int = 0):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {
                "email": email,
                "session_key": session_key,
            }
            find_id = UserDB.filter_invalid_keys(find_id)
            return user_db.unregister_dl_in_sql(find_id=find_id).model_dump(exclude_none=True)

    '''
    description: 往数据库中的space和space_user表中新增space数据
    param {dict} space_db   待创建的space数据
    return {*}
    '''
    @with_exception_handling
    def create_space_tbl(self, space_db: dict):
        with get_db_jw() as db:
            space_repos = JiuwenBaseRepository(db, SpaceDB)
            space_user_repos = JiuwenBaseRepository(db, SpaceUserDB)
            find_id_space = {
                "space_id": get_val_from_dict(space_db, ["space_id"]),
            }
            reg_space = space_repos.register_dl_in_sql(find_id=find_id_space, dl=space_db)
            if reg_space.code != status.HTTP_200_OK:
                return reg_space.model_dump(exclude_none=True)
            find_id_spase_user = {
                "space_id": get_val_from_dict(space_db, ["space_id"]),
                "user_id_str": get_val_from_dict(space_db, ["user_id_str"]),
            }
            reg_space_user = space_user_repos.register_dl_in_sql(find_id=find_id_spase_user,
                                                      dl=space_db)
            if reg_space_user.code != status.HTTP_200_OK:
                # 删除注册成功的space表
                space_repos.unregister_dl_in_sql(find_id=find_id_space)
            return reg_space_user.model_dump(exclude_none=True)
    
    @with_exception_handling
    def update_space_tbl(self, space_db: dict):
        with get_db_jw() as db:
            with db.begin():
                space_repos = JiuwenBaseRepository(db, SpaceDB)
                space_user_repos = JiuwenBaseRepository(db, SpaceUserDB)
                find_id_space = {
                    "space_id": get_val_from_dict(space_db, ["space_id"]),
                }
                reg_space = space_repos.update_dl_in_sql(find_id=find_id_space, update_dl=space_db)
                if reg_space.code != status.HTTP_200_OK:
                    return reg_space.model_dump(exclude_none=True)
                find_id_spase_user = {
                    "space_id": get_val_from_dict(space_db, ["space_id"]),
                    "user_id_str": get_val_from_dict(space_db, ["user_id_str"]),
                }
                return space_user_repos.update_dl_in_sql(
                    find_id=find_id_spase_user,
                    update_dl=space_db
                ).model_dump(exclude_none=True)
    
    @with_exception_handling
    def get_space_tbl(self, space_id: Union[int, str]):
        with get_db_jw() as db:
            space_repos = JiuwenBaseRepository(db, SpaceDB)
            find_id = {
                "space_id": str(space_id),
            }
            return space_repos.get_dl_in_sql(find_id=find_id, return_first_item=True).model_dump(exclude_none=True)
    
    @with_exception_handling
    def get_space_id(self, user_id: Union[int, str]):
        with get_db_jw() as db:
            space_user_repos = JiuwenBaseRepository(db, SpaceUserDB)
            find_id = {
                "user_id_str": str(user_id),
            }
            res = space_user_repos.get_dl_in_sql_with_cols(find_id=find_id, cols_find=['space_id'])
            if res.code != status.HTTP_200_OK:
                return res.model_dump(exclude_none=True)
            res.data = [d['space_id'] for d in res.data]
            return res.model_dump(exclude_none=True)

    @with_exception_handling
    def delete_space_tbl(self, space_id: Union[int, str]):
        with get_db_jw() as db:
            space_repos = JiuwenBaseRepository(db, SpaceDB)
            space_user_repos = JiuwenBaseRepository(db, SpaceUserDB)
            find_id = {
                "space_id": str(space_id),
            }
            del_space = space_repos.unregister_dl_in_sql(find_id=find_id)
            if del_space.code != status.HTTP_200_OK:
                return del_space.model_dump(exclude_none=True)
            return space_user_repos.unregister_dl_in_sql(find_id=find_id).model_dump(exclude_none=True)

    '''
    description: 更新用户密码
    param {str} email 
    param {str} hashed_password 传入加密后的哈希值
    return {*}
    '''
    @with_exception_handling
    def update_user_password(self, email: str, hashed_password: str):
        with get_db_jw() as db:
            user_db = JiuwenBaseRepository(db, UserDB)
            find_id = {"email": email}
            update_data = {
                "password": hashed_password
            }
            return user_db.update_dl_in_sql(
                find_id=find_id,
                update_dl=update_data
            ).model_dump(exclude_none=True)
    
user_repository = UserRepository()



from enum import Enum
from typing import Optional

import re

from pydantic import BaseModel, EmailStr, Field, field_validator


_PASSWORD_MIN_LENGTH = 6
_PASSWORD_MAX_LENGTH = 20
_PASSWORD_SPECIAL_PATTERN = re.compile(r"[^\w]")
_PASSWORD_DIGIT_PATTERN = re.compile(r"\d")
_PASSWORD_LOWER_PATTERN = re.compile(r"[a-z]")
_PASSWORD_UPPER_PATTERN = re.compile(r"[A-Z]")
_PASSWORD_MIN_CLASSES = 2


def _validate_password_strength(password: str) -> str:
    if len(password) < _PASSWORD_MIN_LENGTH:
        raise ValueError("密码长度至少 6 位")
    if len(password) > _PASSWORD_MAX_LENGTH:
        raise ValueError("密码长度不能超过 20 位")
    classes = [
        bool(_PASSWORD_DIGIT_PATTERN.search(password)),
        bool(_PASSWORD_LOWER_PATTERN.search(password)),
        bool(_PASSWORD_UPPER_PATTERN.search(password)),
        bool(_PASSWORD_SPECIAL_PATTERN.search(password)),
    ]
    if sum(classes) < _PASSWORD_MIN_CLASSES:
        raise ValueError("密码需包含数字/小写/大写/特殊字符中至少 2 种")
    return password


class RoleType(Enum):
    COMMON_USER = 0
    SUPER_USER = 1


class UserBase(BaseModel):
    user_id_str: str
    username: str = Field(..., min_length=3, max_length=50)
    user_unique_name: str
    avatar_url: Optional[str] = None
    role_type: RoleType


class UserInfo(UserBase):
    email: EmailStr
    locale: str
    description: Optional[str] = None
    user_create_time: int
    user_update_time: int


class UserTag(BaseModel):
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone_number: Optional[str] = None
    company: Optional[str] = None
    occupation: Optional[str] = None
    skills: Optional[list[str]] = []


class UserDBPd(UserInfo, UserTag):
    session_key: Optional[str] = None
    password: str
    is_active: bool
    refresh_token: Optional[str] = None


class UserResponse(UserInfo, UserTag):
    is_active: bool
    screen_name: str
    app_user_info: Optional[dict] = None

    class Config:
        from_attributes = True


class UserCreate(UserBase):
    password: str = Field(..., min_length=_PASSWORD_MIN_LENGTH, max_length=_PASSWORD_MAX_LENGTH)

    @field_validator("password")
    def validate_password_strength(cls, v: str) -> str:
        return _validate_password_strength(v)


class UserLogin(BaseModel):
    email: EmailStr
    password: str
    role_type: Optional[RoleType] = RoleType.COMMON_USER


class UserUpdate(UserTag):
    username: Optional[str] = Field(None, min_length=3, max_length=50)
    user_unique_name: Optional[str] = Field(None, max_length=100)
    password: Optional[str] = Field(None, min_length=_PASSWORD_MIN_LENGTH, max_length=_PASSWORD_MAX_LENGTH)

    @field_validator("password")
    def validate_password_strength(cls, v: Optional[str]) -> Optional[str]:
        if v is None:
            return v
        return _validate_password_strength(v)


class RefreshTokenRequest(BaseModel):
    refreshToken: str


class SendCodeRequest(BaseModel):
    email: EmailStr


class RegisterRequest(BaseModel):
    email: EmailStr
    password: str
    code: str

    @field_validator("password")
    def validate_password_strength(cls, v: str) -> str:
        return _validate_password_strength(v)


class ResetPasswordRequest(BaseModel):
    email: EmailStr
    code: str
    new_password: str

    @field_validator("new_password")
    def validate_password_strength(cls, v: str) -> str:
        return _validate_password_strength(v)



#!/usr/bin/python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
import os
import textwrap
from typing import Any, Dict

import requests

from openjiuwen_studio.core.executor.component.code_runner.base import CodeRunner


class RemoteCodeRunner(CodeRunner):
    def __init__(self, code_sandbox_url: str) -> None:
        self.code_sandbox_url: str = code_sandbox_url

    async def run(self, code_language: str, code_str: str, timeout: float, params: Dict[str, Any]) -> Any:
        dedented_code = textwrap.dedent(code_str)
        payload = {"language": code_language, "code": dedented_code, "inputs": params, "timeout": timeout}
        response = requests.post(
            self.code_sandbox_url,
            json=payload,
            headers={"Content-Type": "application/json"},
            timeout=timeout  # http超时
        )
        response.raise_for_status()  # 检查 HTTP 错误
        response_data = response.json().get("output")

        if response_data.get("error") is not None:
            raise RuntimeError(f"Sandbox run error: {response_data['error']}")

        result_dict = response_data.get("return")
        return result_dict


remote_code_runner: RemoteCodeRunner = RemoteCodeRunner(
    os.getenv("CODE_SANDBOX_URL", default=""))



"""fix extend mysql text column size limit

Revision ID: 54be202e347e
Revises: 26f51e9528f1
Create Date: 2026-02-04 23:31:22.259889

"""
from typing import Sequence, Union

from alembic import op
from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists
import sqlalchemy as sa
from sqlalchemy.dialects import mysql
from sqlalchemy import inspect

# revision identifiers, used by Alembic.
revision: str = '54be202e347e'
down_revision: Union[str, Sequence[str], None] = '26f51e9528f1'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if column_exists('agent_execution', 'fail_reason'):
        op.alter_column('agent_execution', 'fail_reason',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_comment='the reason for failure',
                   existing_nullable=True)
    
    if not column_exists('agent_publish', 'model_id'):
        op.add_column('agent_publish', sa.Column('model_id', sa.Integer(), nullable=True))
    if not column_exists('agent_publish', 'agent_model_config'):
        op.add_column('agent_publish', sa.Column('agent_model_config', sa.JSON(), nullable=True))
    
    if column_exists('trace_detail', 'input'):
        op.alter_column('trace_detail', 'input',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_comment='输入数据',
                   existing_nullable=True)
    if column_exists('trace_detail', 'output'):
        op.alter_column('trace_detail', 'output',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_comment='输出数据',
                   existing_nullable=True)
    if column_exists('trace_summary', 'fail_reason'):
        op.alter_column('trace_summary', 'fail_reason',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_comment='失败原因',
                   existing_nullable=True)
    if column_exists('workflow_execution', 'fail_reason'):
        op.alter_column('workflow_execution', 'fail_reason',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_comment='the reason for failure',
                   existing_nullable=True)
    if column_exists('workflow_execution_details', 'meta_data'):
        op.alter_column('workflow_execution_details', 'meta_data',
                   existing_type=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   type_=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   existing_nullable=True)
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if column_exists('workflow_execution_details', 'meta_data'):
        op.alter_column('workflow_execution_details', 'meta_data',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_nullable=True)
    if column_exists('workflow_execution', 'fail_reason'):
        op.alter_column('workflow_execution', 'fail_reason',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_comment='the reason for failure',
                   existing_nullable=True)
    if column_exists('trace_summary', 'fail_reason'):
        op.alter_column('trace_summary', 'fail_reason',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_comment='失败原因',
                   existing_nullable=True)
    if column_exists('trace_detail', 'output'):
        op.alter_column('trace_detail', 'output',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_comment='输出数据',
                   existing_nullable=True)
    if column_exists('trace_detail', 'input'):
        op.alter_column('trace_detail', 'input',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_comment='输入数据',
                   existing_nullable=True)
    
    if column_exists('agent_publish', 'agent_model_config'):
        op.drop_column('agent_publish', 'agent_model_config')
    if column_exists('agent_publish', 'model_id'):
        op.drop_column('agent_publish', 'model_id')
    if column_exists('agent_execution', 'fail_reason'):
        op.alter_column('agent_execution', 'fail_reason',
                   existing_type=sa.Text().with_variant(mysql.LONGTEXT(), 'mysql'),
                   type_=mysql.TEXT(collation='utf8mb4_unicode_ci'),
                   existing_comment='the reason for failure',
                   existing_nullable=True)
    # ### end Alembic commands ###



import os
import sys
from logging.config import fileConfig

from alembic import context
from dotenv import load_dotenv
from sqlalchemy import engine_from_config, pool

# 定义项目根目录
PROJECT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", "..", ".."))

# 将项目根目录添加到 sys.path
sys.path.append(PROJECT_ROOT)

# 加载 .env 文件
dotenv_path = os.path.join(PROJECT_ROOT, ".env")
if os.path.exists(dotenv_path):
    load_dotenv(dotenv_path=dotenv_path)

from openjiuwen_studio.ops.modules.prompt.infra.database import Base
# Import the module that defines the models to register them with Base
from openjiuwen_studio.ops.modules.prompt.infra.repositories import orm_repo

config = context.config

if config.config_file_name is not None:
    fileConfig(config.config_file_name)

target_metadata = Base.metadata


def run_migrations_offline() -> None:
    # Build URL from env vars for offline mode as well
    db_type = os.getenv("DB_TYPE", "mysql")
    if db_type == "sqlite":
        sqlite_db = os.getenv("OPS_SQLITE_DB")
        url = f"sqlite:///{sqlite_db}"
    else:
        db_user = os.getenv("DB_USER")
        db_password = os.getenv("DB_PASSWORD")
        db_host = os.getenv("DB_HOST")
        db_port = os.getenv("DB_PORT")
        db_name = os.getenv("OPS_DB_NAME")
        
        if all([db_user, db_password, db_host, db_port, db_name]):
            url = f"mysql+pymysql://{db_user}:{db_password}@{db_host}:{db_port}/{db_name}"
        else:
            # Fallback to .ini value if env vars missing
            url = config.get_main_option("sqlalchemy.url")
    
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online() -> None:
    section = config.get_section(config.config_ini_section, {})
    
    # Override sqlalchemy.url with environment variables
    db_type = os.getenv("DB_TYPE", "mysql")
    if db_type == "sqlite":
        sqlite_db = os.getenv("OPS_SQLITE_DB")
        url = f"sqlite:///{sqlite_db}"
        section["sqlalchemy.url"] = url
        render_as_batch = True
    else:
        db_user = os.getenv("DB_USER")
        db_password = os.getenv("DB_PASSWORD")
        db_host = os.getenv("DB_HOST")
        db_port = os.getenv("DB_PORT")
        db_name = os.getenv("OPS_DB_NAME")
        
        if all([db_user, db_password, db_host, db_port, db_name]):
            url = f"mysql+pymysql://{db_user}:{db_password}@{db_host}:{db_port}/{db_name}"
            section["sqlalchemy.url"] = url
        render_as_batch = False

    connectable = engine_from_config(
        section,
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection, 
            target_metadata=target_metadata,
            render_as_batch=render_as_batch
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
from typing import Dict

from fastapi import APIRouter, Depends, HTTPException, status
from openjiuwen.core.common.logging import logger
from pydantic import ValidationError

from openjiuwen_studio.core.common import dsl
from openjiuwen_studio.core.manager.login_manager.user import get_current_user
from openjiuwen_studio.routers.common import handle_response
import openjiuwen_studio.core.manager.plugin as mgr
# 插件相关模型
from openjiuwen_studio.schemas.plugin import (
    PluginCreate, PluginId, PluginInfoResponse, PluginApiBase,
    PluginApiInfoResponse, PluginListTool, PluginApiInfo, PluginApiInfoCreate, PluginListResponse,
    PluginList, PluginInfo, PluginToolId, ToolId, PluginCodeBase,
    PluginCodeInfo, PluginCodeInfoResponse, PluginPublishResponse, PluginPublish, PluginPublishListResponse,
    PluginPublishInfoResponse
)
from openjiuwen_studio.schemas.common import ResponseModel

plugin_router = APIRouter()


@plugin_router.post("/create", response_model=ResponseModel[PluginId])
async def plugin_create(
        request: PluginCreate,
        current_user: dict = Depends(get_current_user)
):
    """
    Create a plugin

    Args:
        request: Plugin creation request data
        current_user: Current user information

    Returns:
        ResponseModel[PluginId]: Creation result
    """
    try:
        logger.info(f"🔧 Plugin create start")
        logger.info(f"   Request data: {request.model_dump()}")
        logger.info(f"   User: {current_user.get('email', 'unknown')}")
        res = mgr.plugin_create(request, current_user)
        logger.info(f"✅ Plugin create successful")
        return handle_response(res)
    except ValidationError as e:
        logger.error(f"❌ Plugin create validation failed")
        logger.error(f"   Errors: {e.errors()}")
        logger.error(f"   Request data: {request.model_dump() if hasattr(request, 'model_dump') else 'N/A'}")
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail=f"Request validation failed: {e.errors()}"
        ) from e
    except Exception as e:
        logger.error(f"❌ Plugin create failed with unexpected error: {str(e)}")
        import traceback
        logger.error(f"   Traceback: {traceback.format_exc()}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Internal server error: {str(e)}"
        ) from e


@plugin_router.post("/get", response_model=ResponseModel[PluginInfoResponse])
async def plugin_get(
        request: PluginId,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin information

    Args:
        request: Request data containing plugin ID
        current_user: Current user information

    Returns:
        ResponseModel[PluginInfoResponse]: Plugin information
    """
    try:
        logger.info(f"plugin get start")
        res = mgr.plugin_get(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/update", response_model=ResponseModel[Dict])
async def plugin_update(
        request: PluginInfo,
        current_user: dict = Depends(get_current_user)
):
    """
    Update plugin information

    Args:
        request: Plugin update information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Update result
    """
    try:
        logger.info(f"plugin update start")
        res = mgr.plugin_update(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/delete", response_model=ResponseModel[Dict])
async def plugin_delete(
        request: PluginId,
        current_user: dict = Depends(get_current_user)
):
    """
    Delete a plugin

    Args:
        request: Request data containing plugin ID
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Deletion result
    """
    try:
        logger.info(f"plugin delete start")
        res = mgr.plugin_delete(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/list", response_model=ResponseModel[PluginListResponse])
async def plugin_list(
        request: PluginList,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin list

    Args:
        request: Plugin list request parameters
        current_user: Current user information

    Returns:
        ResponseModel[PluginListResponse]: Plugin list result
    """
    try:
        logger.info(f"plugin list start")
        res = mgr.plugin_list(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/convert", response_model=ResponseModel[dsl.Plugin])
async def plugin_convert(
        request: ToolId,
        current_user: dict = Depends(get_current_user)
):
    """
    Convert plugin information

    Args:
        request: Tool ID information
        current_user: Current user information

    Returns:
        ResponseModel[dsl.Plugin]: Converted plugin data
    """
    try:
        logger.info(f"plugin convert start")
        res = mgr.plugin_convert(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/publish", response_model=ResponseModel[PluginPublishResponse])
async def plugin_publish(
        request: PluginPublish,
        current_user: dict = Depends(get_current_user)
):
    """
    Publish a plugin

    Args:
        request: Plugin publish request parameters
        current_user: Current user information

    Returns:
        ResponseModel[PluginPublishResponse]: Publish result
    """
    try:
        logger.info(f"plugin publish start")
        res = mgr.plugin_publish(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        logger.error(f"plugin publish failed, error: {e.errors()}")
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/publish_get", response_model=ResponseModel[PluginPublishInfoResponse])
async def plugin_publish_get(
        request: PluginId,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve published plugin information

    Args:
        request: Request data containing plugin ID
        current_user: Current user information

    Returns:
        ResponseModel[PluginInfoResponse]: Plugin information
    """
    try:
        logger.info(f"publish plugin get start")
        res = mgr.plugin_publish_get(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/publish_list", response_model=ResponseModel[PluginPublishListResponse])
async def plugin_publish_list(
        request: PluginId,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve published plugin list

    Args:
        request: Published plugin list request parameters
        current_user: Current user information

    Returns:
        ResponseModel[PluginPublishInfoResponse]: Published plugin information list
    """
    try:
        logger.info(f"publish plugin list start")
        res = mgr.plugin_publish_list(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        logger.error(f"publish plugin list failed, error: {e.errors()}")
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/publish_delete", response_model=ResponseModel[Dict])
async def plugin_publish_delete(
        request: PluginId,
        current_user: dict = Depends(get_current_user)
):
    """
    Delete a published plugin

    Args:
        request: Request data containing plugin ID
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Deletion result
    """
    try:
        logger.info(f"publish plugin delete start")
        res = mgr.plugin_publish_delete(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/create_api", response_model=ResponseModel[Dict])
async def plugin_create_api(
        request: PluginApiInfoCreate,
        current_user: dict = Depends(get_current_user)
):
    """
    Create plugin API

    Args:
        request: Plugin API creation information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Creation result
    """
    try:
        logger.info(f"🔧 Plugin create_api start")
        logger.info(f"   Request data: {request.model_dump()}")
        logger.info(f"   User: {current_user.get('email', 'unknown')}")
        res = mgr.plugin_create_api(request, current_user)
        logger.info(f"✅ Plugin create_api successful")
        return handle_response(res)
    except ValidationError as e:
        logger.error(f"❌ Plugin create_api validation failed")
        logger.error(f"   Errors: {e.errors()}")
        logger.error(f"   Request: {request.model_dump() if hasattr(request, 'model_dump') else request}")
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail=f"Request validation failed: {e.errors()}"
        ) from e
    except Exception as e:
        logger.error(f"❌ Plugin create_api error: {str(e)}")
        import traceback
        logger.error(f"   Traceback: {traceback.format_exc()}")
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e)) from e


@plugin_router.post("/update_api", response_model=ResponseModel[Dict])
async def plugin_update_api(
        request: PluginApiInfo,
        current_user: dict = Depends(get_current_user)
):
    """
    Update plugin API

    Args:
        request: Plugin API update information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Update result
    """
    try:
        logger.info(f"🔧 Plugin update_api start")
        logger.info(f"   Request data: {request.model_dump()}")
        logger.info(f"   User: {current_user.get('email', 'unknown')}")
        res = mgr.plugin_update_api(request, current_user)
        logger.info(f"✅ Plugin update_api successful")
        return handle_response(res)
    except ValidationError as e:
        logger.error(f"❌ Plugin update_api validation failed")
        logger.error(f"   Errors: {e.errors()}")
        logger.error(f"   Request: {request.model_dump() if hasattr(request, 'model_dump') else request}")
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail=f"Request validation failed: {e.errors()}"
        ) from e
    except Exception as e:
        logger.error(f"❌ Plugin update_api error: {str(e)}")
        import traceback
        logger.error(f"   Traceback: {traceback.format_exc()}")
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=str(e)) from e


@plugin_router.post("/delete_api", response_model=ResponseModel[Dict])
async def plugin_delete_api(
        request: PluginToolId,
        current_user: dict = Depends(get_current_user)
):
    """
    Delete plugin API

    Args:
        request: Plugin API information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Deletion result
    """
    try:
        logger.info(f"plugin delete_api start")
        res = mgr.plugin_delete_tool(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/get_api", response_model=ResponseModel[PluginApiInfoResponse])
async def plugin_get_api(
        request: PluginToolId,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin API

    Args:
        request: Plugin API information
        current_user: Current user information

    Returns:
        ResponseModel[PluginApiInfoResponse]: Plugin API details
    """
    try:
        logger.info(f"plugin get_api start")
        res = mgr.plugin_get_api(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/list_api", response_model=ResponseModel[PluginApiInfoResponse])
async def plugin_list_api(
        request: PluginListTool,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin API list

    Args:
        request: Plugin list request parameters
        current_user: Current user information

    Returns:
        ResponseModel[PluginApiInfoResponse]: Plugin API list
    """
    try:
        logger.info(f"plugin list_api start")
        res = mgr.plugin_list_api(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/create_code", response_model=ResponseModel[Dict])
async def plugin_create_code(
        request: PluginCodeBase,
        current_user: dict = Depends(get_current_user)
):
    """
    Create plugin code tool

    Args:
        request: Plugin code tool creation information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Creation result
    """
    try:
        logger.info(f"plugin create_code start")
        res = mgr.plugin_create_code(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/update_code", response_model=ResponseModel[Dict])
async def plugin_update_code(
        request: PluginCodeInfo,
        current_user: dict = Depends(get_current_user)
):
    """
    Update plugin code tool

    Args:
        request: Plugin code tool update information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Update result
    """
    try:
        logger.info(f"plugin update_code start")
        res = mgr.plugin_update_code(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/delete_code", response_model=ResponseModel[Dict])
async def plugin_delete_code(
        request: PluginToolId,
        current_user: dict = Depends(get_current_user)
):
    """
    Delete plugin code tool

    Args:
        request: Plugin code tool information
        current_user: Current user information

    Returns:
        ResponseModel[Dict]: Deletion result
    """
    try:
        logger.info(f"plugin delete_code start")
        res = mgr.plugin_delete_tool(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/get_code", response_model=ResponseModel[PluginCodeInfoResponse])
async def plugin_get_code(
        request: PluginToolId,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin code tool

    Args:
        request: Plugin code tool information
        current_user: Current user information

    Returns:
        ResponseModel[PluginCodeInfoResponse]: Plugin code tool details
    """
    try:
        logger.info(f"plugin get_code start")
        res = mgr.plugin_get_code(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/list_code", response_model=ResponseModel[PluginCodeInfoResponse])
async def plugin_list_code(
        request: PluginListTool,
        current_user: dict = Depends(get_current_user)
):
    """
    Retrieve plugin code tool list

    Args:
        request: Plugin list request parameters
        current_user: Current user information

    Returns:
        ResponseModel[PluginCodeInfoResponse]: Plugin code tool list
    """
    try:
        logger.info(f"plugin list_code start")
        res = mgr.plugin_list_code(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e


@plugin_router.post("/get_market", response_model=ResponseModel[str])
async def plugin_read_json_file(
    request: PluginList,
    current_user: dict = Depends(get_current_user)
):
    """
    Read JSON file content and return as JSON string (non-frontend interface)

    Args:
        request: Request data containing space ID
        current_user: Current user information

    Returns:
        ResponseModel[str]: JSON file content as string
    """
    try:
        logger.info(f"plugin read json file start")
        res = mgr.plugin_read_market_json(request, current_user)
        return handle_response(res)
    except ValidationError as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Request validation failed") from e



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from typing import List

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.common import dsl
from openjiuwen_studio.core.common.dsl import ComponentType
from openjiuwen_studio.core.database import SessionLocal
from openjiuwen_studio.core.exceptions import ModelApiKeyDecryptError
from openjiuwen_studio.core.manager.convertor.components.common import input_params_convert, outputs_convert
from openjiuwen_studio.core.manager.model_manager.managers import ModelConfigManager
from openjiuwen_studio.core.manager.model_manager.utils import SecurityUtils
from openjiuwen_studio.schemas.model_config import ModelParameters
from openjiuwen_studio.schemas.node import Node, Inputs, Outputs, Edge

DEFAULT_USER_CONTENT = "\n        {{user_prompt}}\n\n        当前可供选择的功能分类如下：\n        {{category_info}}\n\n        用户与助手的对话历史： \n        {{chat_history}}\n\n        当前输入：{{input}}\n\n        请根据当前输入和对话历史分析并输出最适合的功能分类。输出格式为JSON：\n         {\"class\": \"分类xx\"}\n        如果没有合适的分类，请输出 {{default_class}}。\n"


def _intent_inputs_convert(inputs: Inputs) -> dict:
    if inputs is None:
        raise TypeError("inputs is none")

    input_parameters = inputs.input_parameters
    if input_parameters is None:
        raise TypeError("input_parameters is empty")

    converted_inputs = input_params_convert(input_parameters)
    logger.info(f"converted inputs: {converted_inputs}")

    return converted_inputs


def _intent_outputs_convert(outputs: Outputs) -> dict:
    if outputs is None:
        raise TypeError("outputs is none")

    converted_outputs = outputs_convert(outputs)
    logger.info(f"converted outputs: {converted_outputs}")

    return converted_outputs


def _intent_configs_model_convert(model_id: int, space_id: str) -> dsl.ModelConfig:
    db = SessionLocal()
    manager = ModelConfigManager(db)
    model = manager.get_config_by_id(model_id, space_id)

    if model.api_key:
        try:
            model.api_key = SecurityUtils().decrypt_api_key(model.api_key)
        except Exception as e:
            raise ModelApiKeyDecryptError(f"API key decryption failed: {str(e)}") from e

    param = ModelParameters(**model.parameters)

    return dsl.ModelConfig(
        model_client_config=dsl.ModelClientConfig(
            client_provider=model.provider,
            api_key=model.api_key,
            api_base=model.base_url,
            timeout=model.timeout
        ),
        request_config=dsl.ModelRequestConfig(
            model_name=model.model_type,
            temperature=param.temperature,
            top_p=param.top_p,
        )
    )


def _intent_configs_convert(inputs: Inputs, space_id: str) -> dict:
    if inputs is None:
        raise TypeError("inputs is none")

    llm_params = inputs.llm_param
    if llm_params is None:
        raise TypeError("llm_param is none")

    # 提取用户设置的 prompt
    user_prompt = ""
    if llm_params.prompt and llm_params.prompt.content:
        user_prompt = llm_params.prompt.content

    intents = inputs.intents
    category_list: list[str] = []
    category_name_list: list[str] = []
    index: int = 1
    for intent in intents:
        category_index = f"分类{index}"
        category_list.append(category_index)
        category_name_list.append(intent.name)
        index += 1

    model = _intent_configs_model_convert(int(llm_params.model.id), space_id)
    enable_history = inputs.enable_history

    converted_configs = dsl.IntentDetectionConfig(
        user_prompt=user_prompt,  # 添加 user_prompt 字段
        category_list=category_list,
        category_name_list=category_name_list,
        model=model,
        enable_history=enable_history,
    )

    return converted_configs.model_dump()


def intent_convert(node: Node, space_id: str) -> dsl.Component:
    try:
        data = node.data
        converted_inputs = _intent_inputs_convert(data.inputs)
        converted_outputs = _intent_outputs_convert(data.outputs)
        converted_configs = _intent_configs_convert(data.inputs, space_id)

        convert_branches: List[dsl.Branch] = []

        convert_branches.append(dsl.Branch(
            branch_id=data.inputs.default_intent
        ))
        for intent in data.inputs.intents:
            convert_branches.append(dsl.Branch(
                branch_id=intent.id
            ))

        return dsl.Component(
            id=node.id,
            name=data.title,
            type=ComponentType.COMPONENT_TYPE_INTENT,
            type_version="1.0.0",
            inputs=converted_inputs,
            outputs=converted_outputs,
            configs=converted_configs,
            branches=convert_branches,
        )
    except Exception as e:
        raise ValueError(f"Failed to convert intent node: {str(e)}") from e



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

"""
Plugin module for executor components.
"""

from .plugin_mgr import PluginManager
from .plugin_tools import ServiceTool, CodeTool

__all__ = [
    "PluginManager",
    "ServiceTool",
    "CodeTool"
]


from datetime import datetime
from enum import Enum
from typing import Any, Dict, Optional

from pydantic import BaseModel, Field

from openjiuwen_studio.core.common.dsl import BaseInfo


class ExecuteResponseType(str, Enum):
    Trace = "trace"  # 节点trace消息
    Node = "node"  # 节点内主动流式输出
    Workflow = "workflow"  # 工作流消息
    Agent = "agent"  # Agent 消息
    Interaction = "interaction"  # Interaciton消息
    Plugin = "plugin"


class ExecuteStatus(str, Enum):
    Start = "start"
    Finish = "finish"
    Interrupted = "interrupted"
    Agent = "agent"


class ExecuteResponse(BaseModel):
    type: ExecuteResponseType = Field(ExecuteResponseType.Trace)
    payload: dict = Field(default_factory=dict)

    model_config = {
        "use_enum_values": True,  # 序列化时输出枚举值而非对象
        "json_encoders": {ExecuteResponseType: lambda v: v.value,
                          ExecuteStatus: lambda v: v.value}  # 明确指定枚举序列化方式
    }


class TraceResponse(BaseInfo):
    status: ExecuteStatus = Field(ExecuteStatus.Start)
    inputs: Optional[Dict[str, Any]] = Field(default_factory=dict)
    outputs: Optional[Dict[str, Any]] = Field(default_factory=dict)  # 输出字典
    output_text: Optional[str] = Field("")  # 输出文本
    error: Optional[dict] = Field("")
    start_time: Optional[datetime] = Field(default_factory=datetime.now)
    end_time: Optional[datetime] = Field(default=None)
    parent_id: Optional[str] = Field("")
    loop_index: Optional[int] = Field(None)

    model_config = {
        "use_enum_values": True,  # 序列化时输出枚举值而非对象
        "json_encoders": {ExecuteResponseType: lambda v: v.value,
                          ExecuteStatus: lambda v: v.value}  # 明确指定枚举序列化方式
    }


class InteractionResponse(BaseModel):
    interaction_node: str = Field(default_factory=str)
    interaction_msg: Any = Field(default_factory=str)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import re
from typing import Optional, Dict, Any
from pydantic import BaseModel, Field, field_validator

# 记忆库名称中不允许的特殊字符正则表达式
INVALID_MEMORY_NAME_CHARS = re.compile(r'[<>:"/\\|?*\x00-\x1f]')


def validate_memory_name(name: str) -> str:
    """验证记忆库名称，不允许包含特殊字符"""
    if not name or not name.strip():
        raise ValueError("记忆库名称不能为空")

    trimmed_name = name.strip()

    if trimmed_name != name:
        raise ValueError("记忆库名称不能以空格开头或结尾")

    if INVALID_MEMORY_NAME_CHARS.search(trimmed_name):
        raise ValueError('记忆库名称不能包含以下字符: < > : " / \\ | ? * 以及控制字符')

    return name


class MemoryBaseCreate(BaseModel):
    """创建记忆库请求"""
    space_id: str = Field(..., min_length=1, max_length=100, description="空间ID")
    name: str = Field(..., min_length=1, max_length=100, description="记忆库名称")
    description: str = Field(..., max_length=2000, description="记忆库描述")
    embedding_model_config_id: int = Field(..., description="Embedding 模型配置ID（必填，记忆库创建时选择，后续不可更改）")
    llm_model_config_id: int = Field(..., description="LLM 模型配置ID（必填，记忆库创建时选择)")

    @field_validator('name')
    @classmethod
    def validate_name(cls, v: str) -> str:
        """验证记忆库名称"""
        return validate_memory_name(v)


class MemoryBaseResponseCreate(BaseModel):
    """创建记忆库响应"""
    mdb_id: str = Field(..., description="记忆库ID")

    class Config:
        populate_by_name = True


class MemoryBaseGet(BaseModel):
    """获取/删除记忆库请求"""
    space_id: str = Field(..., min_length=0, max_length=100, description="空间ID")
    mdb_id: str = Field(..., min_length=1, max_length=100, description="记忆库ID")


class MemoryBaseUpdateRequest(BaseModel):
    """更新记忆库请求"""
    space_id: str = Field(..., min_length=1, max_length=100, description="空间ID")
    mdb_id: str = Field(..., min_length=1, max_length=100, description="记忆库ID")
    name: Optional[str] = Field(None, min_length=1, max_length=100, description="新的名字")
    description: Optional[str] = Field(None, description="新的描述")
    llm_model_config_id: Optional[int] = Field(None, description="LLM 模型配置ID)")

    @field_validator('name')
    @classmethod
    def validate_name(cls, v: str) -> str:
        """验证记忆库名称"""
        return validate_memory_name(v)


class MemoryBaseInfo(BaseModel):
    """记忆库信息"""
    mdb_id: str = Field(...)
    space_id: str = Field(..., description="空间ID")
    name: str = Field(..., description="记忆库名称")
    description: str = Field(..., description="记忆库描述")
    embedding_model_config_id: int = Field(..., description="Embedding 模型配置ID")
    llm_model_config_id: int = Field(..., description="LLM 模型配置ID（必填，记忆库创建时选择)")
    create_time: int = Field(..., description="创建时间")
    update_time: int = Field(..., description="更新时间")

    class Config:
        populate_by_name = True


class MemoryBaseListRequest(BaseModel):
    """记忆库列表请求"""
    space_id: str = Field(..., min_length=1, max_length=100, description="空间ID")
    page: int = Field(1, ge=1, description="页码，默认1")
    page_size: int = Field(10, ge=1, le=100, description="每页大小，默认10")


class MemoryBaseListItem(BaseModel):
    """记忆库列表项"""
    mdb_id: str = Field(..., description="记忆库ID")
    space_id: str = Field(..., description="空间ID")
    name: str = Field(..., description="记忆库名称")
    description: str = Field(..., description="记忆库描述")
    embedding_model_config_id: int = Field(..., description="Embedding 模型配置ID")
    llm_model_config_id: int = Field(..., description="LLM 模型配置ID（必填，记忆库创建时选择)")
    created_at: str = Field(..., description="创建时间，格式：YYYY-MM-DD HH:MM:SS")
    updated_at: str = Field(..., description="更新时间，格式：YYYY-MM-DD HH:MM:SS")


class MemoryBaseListResponse(BaseModel):
    """记忆库列表响应"""
    items: list[MemoryBaseListItem] = Field(..., description="记忆库列表")
    total: int = Field(..., description="总记录数")
    page: int = Field(..., description="当前页码")
    size: int = Field(..., description="每页大小")


class MemoryBaseSearchRequest(BaseModel):
    """记忆库查询请求"""
    space_id: str = Field(..., min_length=1, max_length=100, description="空间ID")
    query: str = Field(..., min_length=1, max_length=500,
                       description="查询词（查询词完整出现在记忆库名称或描述中，大小写不敏感）")
    page: Optional[int] = Field(1, ge=1, description="页码，从1开始")
    page_size: Optional[int] = Field(10, ge=1, le=100, description="每页大小，最大100")


class MemoryBaseSearchResponse(BaseModel):
    """记忆库查询响应"""
    memory_bases: list[MemoryBaseListItem] = Field(..., description="匹配的记忆库列表")
    total: int = Field(..., description="总记录数")
    page: int = Field(..., description="当前页码")
    page_size: int = Field(..., description="每页大小")
    total_pages: int = Field(..., description="总页数")


import traceback
from typing import Any
from fastapi import HTTPException

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.core.common.language_thread_context import get_language


def _get_message(zh_msg: str, en_msg: str) -> str:
    """
    根据当前语言环境返回对应的错误消息
    
    Args:
        zh_msg: 中文消息
        en_msg: 英文消息
        
    Returns:
        str: 对应语言的错误消息
    """
    language = get_language()
    if language == 'zh-cn' or language == 'zh':
        return zh_msg
    return en_msg


ERROR_MESSAGE_MAPPING = {
    # 网络相关异常
    'ConnectionError': ("服务连接失败，请稍后重试", "Service connection failed, please try again later"),
    'TimeoutError': ("服务连接超时，请稍后重试", "Service connection timeout, please try again later"),
    # 权限认证相关异常
    'PermissionError': ("权限不足或认证失败", "Insufficient permissions or authentication failed"),
    'AuthenticationError': ("权限不足或认证失败", "Insufficient permissions or authentication failed"),
    # 参数验证相关异常
    'ValueError': ("请求参数格式错误", "Invalid request parameter format"),
    'ValidationError': ("请求参数格式错误", "Invalid request parameter format"),
    # 文件操作相关异常
    'FileNotFoundError': ("文件操作失败", "File operation failed"),
    'IOError': ("文件操作失败", "File operation failed"),
    # 数据库相关异常
    'DatabaseError': ("数据库操作失败", "Database operation failed"),
    'IntegrityError': ("数据库操作失败", "Database operation failed"),
}


ERROR_CODE_MAPPING = {
    # 模型相关错误码 (181xxx)
    181001: ("模型调用失败，请检查模型配置（API Key、Base URL、模型名称等）",
             "Model call failed, please check model configuration (API Key, Base URL, model name, etc.)"),
    181002: ("模型服务配置错误，请检查模型配置",
             "Model service configuration error, please check model configuration"),
    181003: ("模型配置错误，请检查模型参数",
             "Model configuration error, please check model parameters"),
    181004: ("模型调用参数错误，请检查输入参数",
             "Model call parameter error, please check input parameters"),
    181005: ("模型客户端配置无效，请检查模型配置",
             "Model client configuration invalid, please check model configuration"),
    
    # 智能体控制器相关错误码 (123xxx)
    123000: ("智能体控制器调用失败，请检查智能体配置",
             "Agent controller call failed, please check agent configuration"),
    123003: ("智能体控制器运行时错误，请联系管理员",
             "Agent controller runtime error, please contact the administrator"),
    
    # 智能体工具相关错误码 (120xxx)
    120000: ("智能体工具未找到，请检查工具配置",
             "Agent tool not found, please check tool configuration"),
    120001: ("智能体工具执行错误，请检查工具配置",
             "Agent tool execution error, please check tool configuration"),
    120003: ("智能体工作流执行错误，请检查工作流配置",
             "Agent workflow execution error, please check workflow configuration"),
    
    # 提问器组件相关错误码 (101xxx)
    101070: ("提问器输入参数错误，请检查提问器配置",
             "Questioner input parameter error, please check questioner configuration"),
    101071: ("提问器配置错误，请检查提问器参数设置",
             "Questioner configuration error, please check questioner parameter settings"),
    101072: ("提问器输入无效，请检查输入数据格式",
             "Questioner input invalid, please check input data format"),
    101073: ("提问器状态初始化失败，请检查提问器配置",
             "Questioner state initialization failed, please check questioner configuration"),
    101074: ("提问器运行错误：达到最大响应次数仍未获取所有必需信息，请增加最大响应次数或简化问题",
             "Questioner runtime error: Maximum response count reached without obtaining all "
             "required information, please increase max response count or simplify questions"),
    101075: ("提问器调用失败，请检查提问器配置",
             "Questioner call failed, please check questioner configuration"),
    101076: ("提问器执行过程错误，请检查提问器配置",
             "Questioner execution process error, please check questioner configuration"),
}


def _extract_model_error_message(error_message: str) -> str:
    """
    从模型调用错误消息中提取用户友好的错误提示
    
    Args:
        error_message: 原始错误消息
        
    Returns:
        str: 用户友好的错误消息
    """
    error_lower = error_message.lower()
    
    if "302" in error_message or "redirect" in error_lower:
        return _get_message(
            "模型服务地址重定向错误，请检查模型配置中的Base URL是否正确",
            "Model service URL redirect error, please check if the Base URL in model configuration is correct"
        )
    
    if "401" in error_message or "invalid_api_key" in error_lower or "incorrect api key" in error_lower:
        return _get_message(
            "模型API Key无效或已过期，请检查模型配置中的API Key",
            "Model API Key is invalid or expired, please check the API Key in model configuration"
        )
    
    if "403" in error_message or "forbidden" in error_lower or "access denied" in error_lower:
        return _get_message(
            "模型访问权限不足，请检查API Key权限或账户余额",
            "Insufficient model access permissions, please check API Key permissions or account balance"
        )
    
    if "404" in error_message or "not found" in error_lower:
        if "model" in error_lower:
            return _get_message(
                "模型不存在或模型名称错误，请检查模型配置中的模型名称",
                "Model does not exist or model name is incorrect, please check the model name in model configuration"
            )
        return _get_message(
            "模型服务地址错误，请检查模型配置中的Base URL",
            "Model service URL is incorrect, please check the Base URL in model configuration"
        )
    
    if "429" in error_message or "rate limit" in error_lower or "quota" in error_lower:
        return _get_message(
            "模型调用频率超限或配额不足，请稍后重试或检查账户余额",
            "Model call rate limit exceeded or insufficient quota, please try again later or check account balance"
        )
    
    server_errors = ("500", "502", "503")
    if any(code in error_message for code in server_errors) or "internal server error" in error_lower:
        return _get_message(
            "模型服务异常，请稍后重试或联系服务提供商",
            "Model service error, please try again later or contact the service provider"
        )
    
    if "timeout" in error_lower or "timed out" in error_lower:
        return _get_message(
            "模型调用超时，请检查网络连接或增加超时时间配置",
            "Model call timeout, please check network connection or increase timeout configuration"
        )
    
    if "connection" in error_lower or "network" in error_lower or "dns" in error_lower:
        return _get_message(
            "模型服务连接失败，请检查网络连接和模型服务地址",
            "Model service connection failed, please check network connection and model service URL"
        )
    
    if "ssl" in error_lower or "certificate" in error_lower:
        return _get_message(
            "模型服务SSL证书验证失败，请检查安全配置",
            "Model service SSL certificate verification failed, please check security configuration"
        )
    
    return None


def log_exception(e: Exception):
    """记录异常详情到日志，包含完整的堆栈信息"""
    logger.error(f"Exception: {repr(e)}")
    stack_frames = traceback.extract_tb(e.__traceback__)
    for frame in stack_frames:
        logger.debug(
            f"File \"{frame.filename}\", line {frame.lineno}, in {frame.name}")


def get_safe_error_message(e: Exception, custom_message: str = None) -> str:
    """
    获取安全的错误消息，避免敏感信息泄露

    Args:
        e: 异常对象
        custom_message: 自定义错误消息

    Returns:
        str: 安全的错误消息
    """
    if settings.debug:
        # 开发环境：返回详细错误信息
        return f"{custom_message}: {str(e)}" if custom_message else str(e)
    # 生产环境：返回通用错误消息
    if custom_message:
        return custom_message

    # 根据异常类型返回相应的通用消息
    error_type = type(e).__name__
    
    # 优先尝试从错误消息中提取模型相关的具体错误（优先级最高）
    error_message = str(e)
    model_error = _extract_model_error_message(error_message)
    if model_error:
        return model_error
    
    # 检查错误码映射（针对BaseError等有code属性的异常）
    if hasattr(e, 'code'):
        error_code = getattr(e, 'code', None)
        if error_code and error_code in ERROR_CODE_MAPPING:
            # 使用预定义的错误码映射（支持中英双语）
            zh_msg, en_msg = ERROR_CODE_MAPPING[error_code]
            return _get_message(zh_msg, en_msg)
    
    # 检查异常对象是否有code属性（如BaseError）
    if hasattr(e, 'code'):
        error_code = getattr(e, 'code', None)
        # 如果有具体的错误码，尝试提取错误消息中的有用信息
        if error_code and error_code != -1:
            # 对于已知的业务错误，返回原始错误消息
            if hasattr(e, 'message') and e.message:
                return e.message

    # 根据异常类型返回相应的通用消息（支持中英双语）
    if error_type in ERROR_MESSAGE_MAPPING:
        zh_msg, en_msg = ERROR_MESSAGE_MAPPING[error_type]
        return _get_message(zh_msg, en_msg)
    
    return _get_message(
        "系统内部错误，请联系管理员",
        "System internal error, please contact the administrator"
    )


def handle_http_exception(e: Exception, custom_message: str = None, status_code: int = 500) -> Exception:
    """
    处理HTTP异常，返回安全的异常信息

    Args:
        e: 原始异常
        custom_message: 自定义错误消息
        status_code: HTTP状态码

    Returns:
        Exception: 处理后的HTTPException
    """
    # 记录完整异常信息到日志
    log_exception(e)

    # 获取安全的错误消息
    safe_message = get_safe_error_message(e, custom_message)

    return HTTPException(status_code=status_code, detail=safe_message)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

from typing import Any, List

from openjiuwen.core.common.logging import logger
from openjiuwen.core.workflow import BranchComponent
from openjiuwen.core.workflow.components.condition.condition import AlwaysTrue

from openjiuwen_studio.core.common.dsl import Connection, Branch
from openjiuwen_studio.core.executor.component.compile import util
from openjiuwen_studio.core.executor.component.compile.base_comp_compiler import BaseCompCompiler
from openjiuwen_studio.core.common.exceptions import JiuWenExecuteException
from openjiuwen_studio.core.common.status_code import StatusCode


class BranchCompCompiler(BaseCompCompiler):
    """Branch Component Compiler"""

    def __init__(
            self,
            node_id: str,
            branch_comp_branches: List[Branch],
            workflow_connections: List[Connection]
    ) -> None:
        super().__init__()
        self.component_id = node_id
        self.branch_list = branch_comp_branches
        self.workflow_connections = workflow_connections

    def compile(self) -> BranchComponent:
        """Compile branch component"""
        branch_comp = BranchComponent()
        if not self.branch_list:
            logger.error(
                f"The branches in component id: {self.component_id} is empty, please check!"
            )
            raise JiuWenExecuteException(StatusCode.BRANCH_COMPONENT_COMPILE_ERROR.code,
                                         StatusCode.BRANCH_COMPONENT_COMPILE_ERROR.errmsg.format(
                                             msg=f"Failed to add branches: Component [{self.component_id}] branch configuration is empty"),
                                         node_id=self.component_id)

        for branch_info in self.branch_list:
            targets = util.get_targets(self.component_id, branch_info.branch_id, self.workflow_connections)
            logger.info(f"[DEBUG] 调用 branch_comp.add_branch: component_id={self.component_id}, condition={branch_info.bool_expression if branch_info.bool_expression else AlwaysTrue()}, target={targets}, branch_id={branch_info.branch_id}")
            branch_comp.add_branch(
                condition=branch_info.bool_expression if branch_info.bool_expression else AlwaysTrue(),
                target=targets, branch_id=branch_info.branch_id)
        return branch_comp



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2026. All rights reserved.
"""add obs_name to knowledge_base_document (OBS support for KB)

Revision ID: a1b2c3d4e5f6
Revises: 072ac1293a02
Create Date: 2026-03-02

"""
from typing import Sequence, Union

from alembic import op
from openjiuwen_studio.core.database.migration_utils import column_exists, table_exists
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision: str = "a1b2c3d4e5f6"
down_revision: Union[str, Sequence[str], None] = "072ac1293a02"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Add obs_name column for OBS object path (distributed KB)."""
    if not table_exists("knowledge_base_document"):
        return
    if column_exists("knowledge_base_document", "obs_name"):
        return
    # Existing rows get empty string; new uploads will set obs_name (or local path convention).
    op.add_column(
        "knowledge_base_document",
        sa.Column(
            "obs_name",
            sa.String(length=1000),
            nullable=False,
            server_default=sa.text("''"),
            comment="OBS 存储路径在存储桶中",
        ),
    )


def downgrade() -> None:
    """Remove obs_name column."""
    if not table_exists("knowledge_base_document"):
        return
    if not column_exists("knowledge_base_document", "obs_name"):
        return
    op.drop_column("knowledge_base_document", "obs_name")



"""
Trace Detail Pydantic 模型定义

统一的追踪数据模型，替代原有的workflow_trace和agent_trace分离设计
"""

from typing import Optional, Dict, Any
from pydantic import BaseModel, Field, ConfigDict


class TraceDetail(BaseModel):
    """统一的 Trace Detail 模型"""
    space_id: str = Field("", max_length=100)
    business_id: str = Field("", max_length=100)
    business_type: str = Field("", max_length=20)
    trace_id: str = Field("", max_length=100)
    span_id: str = Field("", max_length=100)
    span_type: str = Field("", max_length=512)
    span_name: str = Field("", max_length=255)
    parent_span_id: Optional[str] = Field(None, max_length=100)
    method: Optional[str] = Field(None, max_length=10)
    psm: Optional[str] = Field(None, max_length=100)
    logid: Optional[str] = Field(None, max_length=100)
    platform_type: Optional[str] = Field(None, max_length=50)
    start_time_micros: Optional[int] = Field(0, description="开始时间微秒戳")
    end_time_micros: Optional[int] = Field(0, description="结束时间微秒戳")
    status_code: Optional[str] = Field(None, max_length=20)
    input: Optional[str] = Field(None)
    output: Optional[str] = Field(None)
    attributes: Optional[Dict[str, Any]] = Field({})

    model_config = ConfigDict(
        from_attributes=True,
        str_strip_whitespace=True,
    )


# clients/deepsearch_http.py
import json
import logging
from typing import AsyncGenerator, Optional, Dict, Any
import httpx
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.core.common.status_code import StatusCode
from openjiuwen_studio.core.common.exceptions import DeepSearchClientError

logger = logging.getLogger(__name__)


class LazyDeepSearchHttpClient:
    """
    通用的、懒加载的 DeepSearch Agent HTTP 客户端。
    支持任意 GET/POST，流式/非流式，由调用方指定。
    """
    _instance: Optional["LazyDeepSearchHttpClient"] = None
    _client: Optional[httpx.AsyncClient] = None
    _initialized: bool = False

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    async def _initialize(self) -> bool:
        try:
            if not settings.deepsearch_agent_host or not settings.deepsearch_agent_port:
                raise ValueError("DeepSearch agent host/port not configured")

            base_url = f"http://{settings.deepsearch_agent_host}:{settings.deepsearch_agent_port}"
            self._client = httpx.AsyncClient(
                base_url=base_url,
                timeout=httpx.Timeout(
                    timeout=3600.0,
                    connect=10.0
                ),
                limits=httpx.Limits(max_connections=50),
                follow_redirects=True,
                verify=False
            )

            try:
                resp = await self._client.get("/", timeout=5.0)
                resp.raise_for_status()
            except (httpx.ConnectError, httpx.TimeoutException, httpx.HTTPStatusError) as ex:
                raise RuntimeError(f"DeepSearch service unreachable: {ex}") from ex

            self._initialized = True
            logger.info(f"DeepSearch HTTP client ready. Base URL: {base_url}")
            return True
        except Exception as e:
            self._client = None
            self._initialized = False
            logger.error(f"Failed to initialize DeepSearch HTTP client: {e}")
            raise DeepSearchClientError(
                error_code=StatusCode.TASK_SPACE_THIRDPARTY_CLIENT_ERROR.code,
                message=StatusCode.TASK_SPACE_THIRDPARTY_CLIENT_ERROR.errmsg.format(msg=str(e))
            ) from e

    async def request(
        self,
        method: str,
        url: str,
        *,
        jsons: Any = None,
        params: Dict[str, Any] = None,
        headers: Dict[str, str] = None
    ):
        """通用 HTTP 请求方法"""
        if not self._initialized or self._client is None:
            await self._initialize()
        try:
            resp = await self._client.request(method, url, json=jsons, params=params, headers=headers)
            resp.raise_for_status()
            return resp
        except httpx.HTTPStatusError:
            # Let HTTP errors bubble up so caller can handle them meaningfully
            raise
        except Exception as e:
            # 可选：重置状态，允许下次重试
            self._client = None
            self._initialized = False
            raise RuntimeError(f"DeepSearch service call failed: {e}") from e

    async def close(self):
        if self._client:
            await self._client.aclose()
            self._client = None
            self._initialized = False

    async def stream(self, method: str, url: str, **kwargs):
        """用于流式请求，返回异步上下文管理器"""
        if not self._initialized or self._client is None:
            await self._initialize()
        return self._client.stream(method, url, **kwargs)


class DeepSearchAgentClient:
    """
    业务层 client，封装具体 API 调用。
    清晰、可读、可测试。
    """

    def __init__(self):
        self._http = LazyDeepSearchHttpClient()

    async def run_deepsearch_stream(self, payload: Dict[str, Any]) -> AsyncGenerator[str, None]:
        """流式运行深度搜索"""
        async with (await self._http.stream("POST", "/api/v1/agent/deepsearch/run/", json=payload)) as resp:
            resp.raise_for_status()
            async for line in resp.aiter_lines():
                yield line

    async def import_templates(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """导入模板"""
        resp = await self._http.request("POST", "/api/v1/agent/deepsearch/template", jsons=payload)
        return resp.json()

    async def list_templates(self, space_id: str) -> list:
        """列出模板"""
        resp = await self._http.request("GET", f"/api/v1/agent/deepsearch/template/{space_id}")
        return resp.json()

    async def get_templates(self, space_id: str, template_id: int) -> list:
        """列出模板"""
        resp = await self._http.request("GET", f"/api/v1/agent/deepsearch/template/{space_id}/{template_id}")
        return resp.json()

    async def delete_templates(self, space_id: str, template_id: int) -> list:
        """列出模板"""
        resp = await self._http.request("DELETE", f"/api/v1/agent/deepsearch/template/{space_id}/{template_id}")
        return resp.json()

    async def update_templates(self, payload: Dict[str, Any]) -> list:
        """列出模板"""
        resp = await self._http.request("PUT", "/api/v1/agent/deepsearch/template", jsons=payload)
        return resp.json()

    async def create_web_searchs(self, payload: Dict[str, Any]) -> list:
        """创建web搜索引擎"""
        resp = await self._http.request("POST", "/api/v1/agent/deepsearch/web_search/", jsons=payload)
        return resp.json()

    async def get_web_search_engines(self, space_id: str, web_search_engine_id: int) -> list:
        """列出模板"""
        resp = await self._http.request("GET",
                                        f"/api/v1/agent/deepsearch/web_search/{space_id}/{web_search_engine_id}")
        return resp.json()

    async def get_web_search_engine_lists(self, space_id: str) -> list:
        """列出模板"""
        resp = await self._http.request("GET", f"/api/v1/agent/deepsearch/web_search/{space_id}")
        return resp.json()

    async def delete_web_search_engines(self, space_id: str, web_search_engine_id: int) -> list:
        """删除模板"""
        resp = await self._http.request("DELETE",
                                        f"/api/v1/agent/deepsearch/web_search/{space_id}/{web_search_engine_id}")
        return resp.json()

    async def update_web_search_engines(self, payload: Dict[str, Any]) -> list:
        """更新模板"""
        resp = await self._http.request("PUT", "/api/v1/agent/deepsearch/web_search/", jsons=payload)
        return resp.json()

    async def access_web_search_engines(self, space_id: str, web_search_engine_id: int, query: str):
        """测试模板"""
        resp = await self._http.request("POST",
                                        f"/api/v1/agent/deepsearch/web_search/{space_id}/{web_search_engine_id}",
                                        jsons=query)
        return resp.json()

    async def report_converts(self, payload: Dict[str, Any]) -> list:
        """列出模板"""
        resp = await self._http.request("POST", "/api/v1/agent/deepsearch/reports/convert", jsons=payload)
        return resp.json()
